/***************************************************************************
 *   Copyright (C) 2009 by Herbert Roider                                  *
 *   herbert.roider@utanet.at                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *

 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/*
 * crossbow.h
 *  Created on: 30.05.2009
 */

#ifndef CROSSBOW_H_
#define CROSSBOW_H_
#include "ballistic.h"
#include <vector>

namespace arw {

/** not ready implemented:
 Gibt an, wie das Koordinatensystem vom Zielgerät verschoben und gedreht ist zum Armbrustkoordinatensystem.
 trans ist der Vector, der vom Armbrustkoordinatensystemursprung zum Visierursprung
 zeigt.
 rot ist der Vector, der die Drehachse angibt, um die das Visier zum Armbrustkoordiantensystem verdreht ist.
 Das Armbrustkoordinatensystem hat seinen Ursprung am Ende der Führungsbahn. Gedreht ist die x-Achse genau in Pfeilrichtung.
 Wenn das Visier nach unten geneigt ist, dann ist die z-Achse ein negativer Winkel
 Im globalen Koordinatensystem zeigt die x-Achse vom Ursprung des Armbrustcoordinatensystem zum Ziel. Der Ursprung deckt sich mit
 dem Ursprung vom Armbrustkoordinatensystem.
 Wichtig:
 Zuerst kommt die Translation, dann die Rotation
 */
class sight_coordinate_system_origin {
public:
	/**
	 point to the origin of the sight from the crossbow origin (m)
	 */
	utility::vector_3d trans;
	/** angle of the sight [rad]
	 */
	utility::vector_3d rot;
	bool operator ==(const sight_coordinate_system_origin & scso) {
		if (this->trans.x != scso.trans.x)
			return false;
		if (this->trans.y != scso.trans.y)
			return false;
		if (this->trans.z != scso.trans.z)
			return false;

		if (this->rot.x != scso.rot.x)
			return false;
		if (this->rot.y != scso.rot.y)
			return false;
		if (this->rot.z != scso.rot.z)
			return false;
		return true;
	}
};
/**
 * the origin is normally the point where the arrow leaves the crossbow.
 * the x-axis is the arrow-axis.
 * the y-axis is the vertical axis
 *
 *
 */
class crossbow {
public:
	crossbow();
	virtual ~crossbow();
	/**
	 * the position of the sight from the crossbow origin.
	 */
	sight_coordinate_system_origin m_scso;

	/**
	 * the energy of the arrow at release time in Joule.
	 *
	 */
	double m_energy;
	/**
	 * This function calc the flightparam at zerorange. It test only the vertical direction
	 * @param _endparam
	 * @param _beginparam
	 * @param scso
	 * @param sight_setting  the directon vector to the target from the sight system, only the direction is necessary
	 * @param velocity
	 * @return  zero at success, otherwise <> 0
	 */
	int flightparam_at_zero_range(flight_snapshot & _endparam,
			flight_snapshot & _beginparam, utility::vector_3d sight_setting,
			flightparam fp);

	/**
	 * @see flightparam_at_zero_range
	 * @param _endparam
	 * @param _beginparam
	 * @param scso
	 * @param sight_setting
	 * @param velocity
	 * @return
	 */
	int flightparam_at_init_range(flight_snapshot & _endparam,
			flight_snapshot & _beginparam, utility::vector_3d sight_setting,
			flightparam fp);

	/**
	 *  Need the arrow speed,
	 * instead of calculate the speed which the energy and the weight.
	 * additional argument: targetpoint_from_crossbow
	 * @see crossbow#sight_setting
	 *
	 * @param targetpoint_from_sight    the targetpoint from the sight coordinate system.
	 * @param targetpoint_from_crossbow    the targetpoint from the crossbow coordinate system.
	 * @param endparam   function writes in this value the impact parameter (velocity, rotation,..)
	 * @param beginparam function writes in this value the release parameter (velocity, rotation,..)
	 * @param dEntfernung the range to the target
	 * @param fp         the flight parameter
	 * @param v          velocity of the arrow
	 * @return 0 if succeed, otherwise <> 0
	 *
	 */
	int sight_setting(utility::vector_3d & targetpoint_from_sight,
			utility::vector_3d & targetpoint_from_crossbow,
			flight_snapshot & endparam, flight_snapshot & beginparam,
			double dEntfernung, //m
			flightparam fp, double v);
	/**
	 * overloaded function. Need the arrow speed,
	 * instead of calculate the speed which the energy and the weight.
	 * @see crossbow#sight_setting
	 *
	 * @param targetpoint_from_sight    the targetpoint from the sight coordinate system.
	 * @param endparam   function writes in this value the impact parameter (velocity, rotation,..)
	 * @param beginparam function writes in this value the release parameter (velocity, rotation,..)
	 * @param dEntfernung the range to the target
	 * @param fp         the flight parameter
	 * @param v          velocity of the arrow
	 * @return 0 if succeed, otherwise <> 0
	 *
	 */
	int sight_setting(utility::vector_3d & targetpoint_from_sight,
			flight_snapshot & endparam, flight_snapshot & beginparam,
			double dEntfernung, //m
			flightparam fp, double v);

	/**
	 * overloadet function
	 * @deprecated
	 *
	 * @param targetpoint_from_sight    function writes in this value the setting value for the sight.
	 * @param endparam   function writes in this value the impact parameter (velocity, rotation,..)
	 * @param beginparam function writes in this value the release parameter (velocity, rotation,..)
	 * @param dEntfernung the range to the target
	 * @param fp         the flight parameter
	 * @return 0 if succeed, otherwise <> 0
	 *
	 * */
	int sight_setting(utility::vector_3d & targetpoint_from_sight,
			flight_snapshot & endparam, flight_snapshot & beginparam,
			double dEntfernung, //m
			flightparam fp);
	/**
	 * overloaded function.
	 * additional argument: targetpoint_from_crossbow
	 * @see crossbow#sight_setting
	 *
	 *
	 * @param targetpoint_from_sight    the targetpoint from the sight coordinate system.
	 * @param targetpoint_from_crossbow    the targetpoint from the crossbow coordinate system.
	 * @param endparam   function writes in this value the impact parameter (velocity, rotation,..)
	 * @param beginparam function writes in this value the release parameter (velocity, rotation,..)
	 * @param dEntfernung the range to the target
	 * @param fp         the flight parameter
	 * @return 0 if succeed, otherwise <> 0
	 */

	int sight_setting(utility::vector_3d & targetpoint_from_sight,
			utility::vector_3d & targetpoint_from_crossbow,
			flight_snapshot & endparam, flight_snapshot & beginparam,
			double dEntfernung, //m
			flightparam fp);



	int calc_velocity(double & v, //m/s
			utility::vector_3d & scso_rot_, //angle
			double dKlEntf, //m
			double dGrEntf, //m
			utility::vector_3d dEinstellwertKlEntf, //m/m
			utility::vector_3d dEinstellwertGrEntf, //m/m
			utility::vector_3d devitation_small_range, //m (from the targetpoint system, x is zero)
			utility::vector_3d devitation_wide_range, //m (from the targetpoint system, x  is zero)
			flightparam fp);

	int calc_sight_inclination(utility::vector_3d & scso_rot, //angle
			double range, //m
			utility::vector_3d _setting, //m/m
			utility::vector_3d devitation, //m (from the targetpoint system, x is zero)
			flightparam fp);

	int calc_max_range(utility::vector_3d & setting,
			flight_snapshot & begin_fs, flight_snapshot & end_fs,
			flightparam fp);

	int calc_flight_table(std::vector<flight_snapshot> &global_table,
			std::vector<flight_snapshot> &crossbow_table, std::vector<
					flight_snapshot> &sight_table,
			std::vector<flight_snapshot> &line_of_sight_table, double max_time, // s
			double max_range, // m in x-Axis
			double zero_range, flightparam fp);
};

}
; //arw
#endif /* CROSSBOW_H_ */
