/***************************************************************************
 *   Copyright (C) 2010 by Herbert Roider   *
 *   herbert.roider@utanet.at   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <ballistic/ballistic.h>
#include <ballistic/utility.h>


#include "Armbrust.h"
#include "arrowmatcher_file.h"
#include <QDebug>
#include <QApplication>

using namespace arw;

/**
* \brief Constructor
*
* \param
*
* \return
* \retval
*
*/
CArmbrust::CArmbrust()
{
	init();
}
/** construct a crossbow with the same values as a, also the guid
 * @param CArmbrust a   the crossbow to copy
 *
 */
CArmbrust::CArmbrust(const CArmbrust &a) : Datastore(a)
{
    //setValue("", a.m_data);
	take_data(a);
}
CArmbrust::CArmbrust(const QString guid) : Datastore(guid)
{
	init();
}

CArmbrust::~CArmbrust()//Destructor
{

}
/** copy all values from a to the current crossbow, eccept the guid!
 * @param CArmbrust a
 *
 */
CArmbrust & CArmbrust::take_data(const CArmbrust & a) {
	   if(this == &a)
	        return *this;

	    m_name = a.m_name;
	    m_energy = a.m_energy;
	    m_scso = a.m_scso;
	    m_Zielgeraet = a.m_Zielgeraet;
	    Datastore::take_data(a);
	    return *this;


}
/**
* @brief read the m_data and write the values to the underlaying crossbow object.
*
*  @param   QString, QVariant
*
*  @return  bool
*
*/
bool CArmbrust::setValue(QString path, QVariant value) {
	bool ok = Datastore::setValue(path, value);
	if(!ok)
		return false;
	UnitConverter uv;
	double v;
	QString unitstr;
	unitstr = this->getValue("energy/unit").toString();
	v = this->getValue("energy/value").toDouble();
	uv.convert_to_SI(v, unitstr);
	m_energy = v;

	unitstr = this->getValue("scso/rot/x/unit").toString();
	v = this->getValue("scso/rot/x/value").toDouble();
	uv.convert_to_SI(v, unitstr);
	m_scso.rot.x = v;

	unitstr = this->getValue("scso/rot/y/unit").toString();
	v = this->getValue("scso/rot/y/value").toDouble();
	uv.convert_to_SI(v, unitstr);
	m_scso.rot.y = v;

	unitstr = this->getValue("scso/rot/z/unit").toString();
	v = this->getValue("scso/rot/z/value").toDouble();
	uv.convert_to_SI(v, unitstr);
	m_scso.rot.z = v;

	unitstr = this->getValue("scso/trans/x/unit").toString();
	v = this->getValue("scso/trans/x/value").toDouble();
	uv.convert_to_SI(v, unitstr);
	m_scso.trans.x = v;

	unitstr = this->getValue("scso/trans/y/unit").toString();
	v = this->getValue("scso/trans/y/value").toDouble();
	uv.convert_to_SI(v, unitstr);
	m_scso.trans.y = v;

	unitstr = this->getValue("scso/trans/z/unit").toString();
	v = this->getValue("scso/trans/z/value").toDouble();
	uv.convert_to_SI(v, unitstr);
	m_scso.trans.z = v;


	m_Zielgeraet.m_graduation.x = 1.0;

	unitstr = this->getValue("sight/screw/y/unit").toString();
	v = this->getValue("sight/screw/y/value").toDouble();
	uv.convert_to_SI(v, unitstr);
	m_Zielgeraet.m_graduation.y = v;

	unitstr = this->getValue("sight/screw/z/unit").toString();
	v = this->getValue("sight/screw/z/value").toDouble();
	uv.convert_to_SI(v, unitstr);
	m_Zielgeraet.m_graduation.z = v;

	QMap<QString, QVariant> map = this->getValue("sight/reticle").toMap();
	m_Zielgeraet.m_marks.clear();
	utility::vector_3d vect;
	for(int j=0; j<100;j++) {
		if( ! map.contains(QString().number(j))) {
			break;
		}

		vect.x = 1.0;

		unitstr = this->getValue("sight/reticle/" + QString().number(j) + "/y/unit").toString();
		v = this->getValue("sight/reticle/" + QString().number(j) + "/y/value").toDouble();
		uv.convert_to_SI(v, unitstr);
		vect.y = v;

		unitstr = this->getValue("sight/reticle/" + QString().number(j) + "/z/unit").toString();
		v = this->getValue("sight/reticle/" + QString().number(j) + "/z/value").toDouble();
		uv.convert_to_SI(v, unitstr);
		vect.z = v;

		m_Zielgeraet.m_marks.push_back(vect);

	 }

	if(this->getValue("sight/type/value").toString() == QString("reticle")) {
		m_Zielgeraet.m_iTyp = BENUTZERDEF;
	}else{
		m_Zielgeraet.m_iTyp = KIMME_U_KORN;
	}

	return true;

}

void CArmbrust::init() {
	errno = 0;
	m_energy = 80.0;
	m_name = "";
	m_scso.rot.x = 0.0;
	m_scso.rot.y = 0.0;
	m_scso.rot.z = -0.01;
	m_scso.trans.x = 0.0;
	m_scso.trans.y = 0.05;
	m_scso.trans.z = 0.0;
	m_Zielgeraet.m_marks.clear();
	utility::vector_3d vect;
	vect.x = 1.0;
	vect.y = 0.0;
	vect.z = 0.0;
	m_Zielgeraet.m_marks.push_back(vect);
	m_Zielgeraet.m_iTyp = BENUTZERDEF;
	vect.x = 1.0;
	vect.y = 0.001;
	vect.z = 0.001;
	m_Zielgeraet.m_graduation = vect;

//return;
    ///////////////////////////////////////
	//
	// Defaultvalues, if not already exists
	//
	this->m_data["title"] = this->m_name;
	QMap<QString, QVariant> trans;
	QMap<QString, QVariant> rot;
	QMap<QString, QVariant> scso;


	QMap<QString, QVariant> trans_x;
	trans_x["value"] = QString().setNum(this->m_scso.trans.x);
	trans_x["status"] = Datastore::preset;
	trans_x["unit"] = QString("SI_metre");
	trans_x["title"] = QString("X");
	trans_x["hidden"] = 0;

	QMap<QString, QVariant> trans_y;
	trans_y["value"] = QString().setNum(this->m_scso.trans.y);
	trans_y["status"] = Datastore::preset;
	trans_y["unit"] = QString("SI_metre");
	trans_y["title"] = QString("Y");
	trans_y["hidden"] = 0;


	QMap<QString, QVariant> trans_z;
	trans_z["value"] = QString().setNum(this->m_scso.trans.z);
	trans_z["status"] = Datastore::preset;
	trans_z["unit"] = QString("SI_metre");
	trans_z["title"] = QString("Z");
	trans_z["hidden"] = 0;


	QMap<QString, QVariant> rot_x;
	rot_x["value"] = QString().setNum(this->m_scso.rot.x * 1000.0);
	rot_x["status"] = Datastore::preset;
	rot_x["unit"] = QString("milliradian");
	rot_x["title"] = QString("X");
	rot_x["hidden"] = 1;

	QMap<QString, QVariant> rot_y;
	rot_y["value"] = QString().setNum(this->m_scso.rot.y * 1000.0);
	rot_y["status"] = Datastore::preset;
	rot_y["unit"] = QString("milliradian");
	rot_y["title"] = QString("Y");
	rot_y["hidden"] = 0;


	QMap<QString, QVariant> rot_z;
	rot_z["value"] = QString().setNum(this->m_scso.rot.z * 1000.0);
	rot_z["status"] = Datastore::preset;
	rot_z["unit"] = QString("milliradian");
	rot_z["title"] = QString("Z");
	rot_z["hidden"] = 0;

	trans["title"] = QCoreApplication::translate("crossbow", "Position");
	trans["hidden"] = 0;
	trans["x"] = trans_x;
	trans["y"] = trans_y;
	trans["z"] = trans_z;

	rot["title"] = QCoreApplication::translate("crossbow", "Angle");
	rot["hidden"] = 0;
	rot["x"] = rot_x;
	rot["y"] = rot_y;
	rot["z"] = rot_z;

	scso["title"] = QCoreApplication::translate("crossbow", "Sight Position");
	scso["hidden"] = 0;
	scso["trans"] = trans;
	scso["rot"] = rot;

	this->m_data["scso"] = scso;

	QMap<QString, QVariant> energy;
	energy["value"] = QString().setNum(this->m_energy);
	energy["status"] = Datastore::preset;
	energy["unit"] = QString("SI_joule");
	energy["title"] = QCoreApplication::translate("crossbow", "energy");
	energy["hidden"] = 0;
	this->m_data["energy"] = energy;

	//if(this)
	QMap<QString, QVariant> sight;
	sight["title"] = QCoreApplication::translate("crossbow", "Sight");
	sight["hidden"] = 0;


	QMap<QString, QVariant> screw;
	screw["title"] = QString("Screw");
	screw["hidden"] = 0;


	QMap<QString, QVariant> screw_y;
	screw_y["value"] = QString().setNum(this->m_Zielgeraet.m_graduation.y * 1000.0);
	screw_y["status"] = Datastore::preset;
	screw_y["unit"] = QString("millimetre_per_metre");
	screw_y["title"] = QString("Y");
	screw_y["hidden"] = 0;

	QMap<QString, QVariant> screw_z;
	screw_z["value"] = QString().setNum(this->m_Zielgeraet.m_graduation.z * 1000.0);
	screw_z["status"] = Datastore::preset;
	screw_z["unit"] = QString("millimetre_per_metre");
	screw_z["title"] = QString("Z");
	screw_z["hidden"] = 0;

	screw["y"] = screw_y;
	screw["z"] = screw_z;

	QMap<QString, QVariant> reticle;
	reticle["title"] = QCoreApplication::translate("crossbow", "reticle");
	reticle["hidden"] = 0;

	std::vector<utility::vector_3d> marks = this->m_Zielgeraet.m_marks;
	for (unsigned int i = 0; i < marks.size(); i++) {
		QMap<QString, QVariant> mark;
		QMap<QString, QVariant> mark_y;
		QMap<QString, QVariant> mark_z;

		mark["title"] = QString().setNum(i );
		mark["hidden"] = 0;


		mark_y["value"] = QString().setNum(marks[i].y * 1000.0);
		mark_y["status"] = (int)Datastore::preset;
		mark_y["unit"] = QString("millimetre_per_metre");
		mark_y["title"] = QString("Y");
		mark_y["hidden"] = 0;

		mark_z["value"] = QString().setNum(marks[i].z * 1000.0);
		mark_z["status"] = (int)Datastore::preset;
		mark_z["unit"] = QString("millimetre_per_metre");
		mark_z["title"] = QString("Z");
		mark_z["hidden"] = 0;

		mark["y"] = mark_y;
		mark["z"] = mark_z;

		reticle[QString().setNum(i)] = mark;
	}

	QMap<QString, QVariant> sight_typ;
	sight_typ["title"] = "type";
	if (this->m_Zielgeraet.m_iTyp == KIMME_U_KORN) {
		sight_typ["value"] = QCoreApplication::translate("crossbow", "screw");
		screw["hidden"] = 0;
		reticle["hidden"] = 1;

	} else {
		sight_typ["value"] = QCoreApplication::translate("crossbow", "reticle");
		screw["hidden"] = 1;
		reticle["hidden"] = 0;
	}
	sight_typ["hidden"] = 1;


	sight["type"] = sight_typ;
	sight["screw"] = screw;
	sight["reticle"] = reticle;

	this->m_data["sight"] = sight;

}


/** generate a xml String of the crossbow data.
 * Useful to save the crossbow to the clipboard.
 * @see fromString to restore the crossbow
 *
 */
QString CArmbrust::toString() {
	QDomDocument domDocument("crossbow");
	QDomElement dom_crossbow;
	domDocument.setContent(
			QString("<?xml version=\"1.0\" encoding=\"UTF-8\"?>"));
	QDomElement root = domDocument.createElement("arrowmatcher");
	root.setAttribute("version", "2.0");
	domDocument.appendChild(root);
	dom_crossbow = domDocument.createElement("crossbow");
	this->toDomElement(dom_crossbow);
	root.appendChild(dom_crossbow);
	//qDebug() << QString("dom Crossbow=") << domDocument.toString(4);


	return domDocument.toString(4);
}
/**
* \brief write the child elements to the dom_crossbow
*
* \param QDomElement& the dom Element where to write the child elements
*
* \return int zero if successful
* \retval
*
*/
int CArmbrust::toDomElement(QDomElement &dom_crossbow) {
	arrowmatcher_file af;
	QDomElement dom_element;
	QDomText dom_text;
	QDomDocument domDocument = dom_crossbow.ownerDocument();

	dom_element = domDocument.createElement("guid");
	dom_text = domDocument.createTextNode(this->getGuid().toString());
	dom_element.appendChild(dom_text);
	dom_crossbow.appendChild(dom_element);


	dom_element = domDocument.createElement("map");
	dom_element.setAttribute("name", "data");
	af.writeMap(dom_element, domDocument, this->m_data);
	dom_crossbow.appendChild(dom_element);


	return 0;
}
/**
* \brief read the dom_crossbow Element and write the data to the local CArmbrust
*
* \param const QDomElement&
*
* \return int zero, if successful
* \retval
*
*/
int CArmbrust::fromDomElement(const QDomElement &dom_crossbow) {
	arrowmatcher_file af;
	CArmbrust temp(QUuid(dom_crossbow.firstChildElement("guid").text()));

	QDomElement dom_element = dom_crossbow.firstChildElement("map");
	while (!dom_element.isNull()) {
		if(dom_element.attribute("name") == QString("data")){
			af.readMap(temp.m_data, dom_element);

		}
		dom_element = dom_crossbow.nextSiblingElement("map");

	}


	*this = temp;
	return 0;
}
/** restore the crossbow from a xml-String. Useful to read the crossbow from clipboard
 * @param QString xmlstr
 * @return int zero if successful
 *
 */
int CArmbrust::fromString(const QString xmlstr ) {
	QDomDocument domDocument("crossbow");
	QDomElement dom_crossbow;
	domDocument.setContent(xmlstr);
	QDomElement root = domDocument.documentElement ();
	if( ! root.isElement()) {
		return 1;
	}
	dom_crossbow = root.firstChildElement("crossbow");
	if( ! dom_crossbow.isElement()) {
		return 2;
	}

	this->fromDomElement(dom_crossbow);
	//qDebug() << "arrow xmlstr=" << xmlstr;
	//qDebug() << __LINE__ << __FILE__ << "arrow fromString=" << this->toString();
	return 0;
}

int CArmbrust::fromMapToObject() {
	//this->setValue()
	return 0;


}
