/***************************************************************************
 *   Copyright (C) 2010 by Herbert Roider   *
 *   herbert.roider@utanet.at   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/*
 * ArrowTreeModel.cpp
 *  Created on: 17.11.2009
 */

#include <QtGui>

//#include "ArrowTreeItem.h"
#include "ArrowTreeModel.h"

ArrowTreeModel::ArrowTreeModel(QObject *parent) :
	QAbstractItemModel(parent) {
	m_unitconverter = NULL;
	QList<QString> path;
	//rootData << "Title" << "Summary";
	rootItem = new TreeItem<CPfeil>(NULL);
	rootItem->isRootItem = true;
	rootItem->base_item = NULL;
	rootItem->parentItem = NULL;
	//std::list<CPfeil> list;
	//m_list = list;
	rootItem->base_item_list = &(this->m_list);

	//setupModelData(data.split(QString("\n")), rootItem);


}

ArrowTreeModel::~ArrowTreeModel() {
	delete rootItem;
}

int ArrowTreeModel::columnCount(const QModelIndex &parent) const {
	if (parent.isValid())
		return static_cast<TreeItem<CPfeil>*> (parent.internalPointer())->columnCount();
	else
		return rootItem->columnCount();
}

QVariant ArrowTreeModel::data(const QModelIndex &index, int role) const {
	if (!index.isValid())
		return QVariant();

	TreeItem<CPfeil> *item = static_cast<TreeItem<CPfeil>*> (index.internalPointer());
	if(role == Qt::DecorationRole){
		if(item->parentItem->isRootItem && index.column() == 0){
			QPixmap *image = new QPixmap(":/toolbar/tlb_arrow.png");
			return QVariant(*image);
		}

	}
	if (role != Qt::DisplayRole)
		return QVariant();


	return item->data(index.column());
}

Qt::ItemFlags ArrowTreeModel::flags(const QModelIndex &index) const {
	if (!index.isValid())
		return Qt::NoItemFlags;

	TreeItem<CPfeil> *item = static_cast<TreeItem<CPfeil>*> (index.internalPointer());
	if (item->isRootItem)
		return Qt::ItemIsEnabled;
	// item is an arrow:
	if (item->parentItem == this->rootItem)
		return Qt::ItemIsEnabled | Qt::ItemIsSelectable;

	return Qt::ItemIsEnabled;
}

QVariant ArrowTreeModel::headerData(int section, Qt::Orientation orientation,
		int role) const {
	if (orientation == Qt::Horizontal && role == Qt::DisplayRole) {
		QVariant header;
		switch (section) {
		case 0:
			header = QString(tr("title"));
			break;
		case 1:
			header = QString(tr("value"));
			break;
		case 2:
			header = QString(tr("unit"));
			break;
		}
		return header;
		//return rootItem->data(section);
	}
	return QVariant();
}

QModelIndex ArrowTreeModel::index(int row, int column,
		const QModelIndex &parent) const {
	if (!hasIndex(row, column, parent))
		return QModelIndex();

	TreeItem<CPfeil> *parentItem;

	if (!parent.isValid())
		parentItem = rootItem;
	else
		parentItem = static_cast<TreeItem<CPfeil>*> (parent.internalPointer());

	TreeItem<CPfeil> *childItem = parentItem->child(row);
	if (childItem)
		return createIndex(row, column, childItem);
	else
		return QModelIndex();
}

QModelIndex ArrowTreeModel::parent(const QModelIndex &index) const {
	if (!index.isValid())
		return QModelIndex();

	TreeItem<CPfeil> *childItem =
			static_cast<TreeItem<CPfeil>*> (index.internalPointer());
	if (childItem == NULL)
		return QModelIndex();
	TreeItem<CPfeil> *parentItem = childItem->parent();

	if (parentItem == rootItem)
		return QModelIndex();
	if (parentItem == NULL)
		return QModelIndex();

	return createIndex(parentItem->row(), 0, parentItem);
}

int ArrowTreeModel::rowCount(const QModelIndex &parent) const {
	TreeItem<CPfeil> *parentItem;
	if (parent.column() > 0)
		return 0;

	if (!parent.isValid())
		parentItem = rootItem;
	else
		parentItem = static_cast<TreeItem<CPfeil>*> (parent.internalPointer());

	int i = parentItem->childCount();
	return i;
}


bool ArrowTreeModel::create_data(TreeItem<CPfeil> *child, TreeItem<CPfeil> *parent,
		CPfeil *base_item, const QMap<QString, QVariant> &v) {

	QMap<QString, QVariant>::const_iterator i = v.constBegin();
	child->base_item = base_item;
	child->base_item_list = &m_list;
	child->isRootItem = false;
	child->parentItem = parent;

	QVariant entry = QString(tr("untitled"));
	child->itemData.append(entry);
	child->itemData.append(QString());
	child->itemData.append(QString());

	for (i = v.constBegin(); i != v.constEnd(); i++) {

		if (i.value().type() == QVariant::Map) {
			if( ! i.value().toMap()["hidden"].toInt()  ) {
				TreeItem<CPfeil> *new_child = new TreeItem<CPfeil>(child);
				if (create_data(new_child, child, base_item, i.value().toMap())) {
					child->childItems.append(new_child);
				}
			}
		} else {
			if (i.key() == QString("title")) {
				if (i.value().toString().length() > 0)
					child->itemData[0] = this->trUtf8(i.value().toString().toUtf8() );

			}
			if (i.key() == QString("value")) {
				child->itemData[1] = i.value();
			}
			if (i.key() == QString("unit")) {
				if(m_unitconverter != NULL)
				{
					Unit unit;
					m_unitconverter->get_unit(unit, i.value().toString());
					child->itemData[2] = unit.m_symbol;
				}
			}

		}

	}
	return true;
}

void ArrowTreeModel::write_item(int &depth, TreeItem<CPfeil> *item) {
	TreeItem<CPfeil> *child;
	for (int i = 0; i < item->childItems.size(); i++) {
		child = item->childItems.value(i);
		int child_depth = depth + 1;
		write_item(child_depth, child);
	}

}
CPfeil* ArrowTreeModel::getItem(QModelIndex index) {
	if (!index.isValid())
		return NULL;

	TreeItem<CPfeil> *item = static_cast<TreeItem<CPfeil>*> (index.internalPointer());
	if (item == this->rootItem)
		return NULL;
	return item->base_item;

}
bool ArrowTreeModel::setItem(QModelIndex index, CPfeil &a) {
	if (!index.isValid())
		return false;

	TreeItem<CPfeil> *item = static_cast<TreeItem<CPfeil>*> (index.internalPointer());
	if (item == this->rootItem)
		return false;
	*(item->base_item) = a;
	QModelIndex begin = createIndex(item->row(), 0, rootItem);
	QModelIndex end = createIndex(item->row(), item->columnCount() - 1,
			rootItem);

	emit
	dataChanged(begin, end);
	return true;

}
bool ArrowTreeModel::setItem(CPfeil &a) {
	if (this->rootItem == NULL)
		return false;
	TreeItem<CPfeil> *item = NULL;
	for (int i = 0; i < this->rootItem->childItems.size(); i++) {
		item = rootItem->childItems.value(i);
		if (item->base_item->getGuid() == a.getGuid()) {
			*(item->base_item) = a;
			item->childItems.clear();
			this->create_data(item, rootItem, item->base_item, item->base_item->m_data);
			//TreeItem<CPfeil> *child = this->create_item(item->arrow, item, item->arrow->m_data);

			QModelIndex begin = createIndex(item->row(), 0, rootItem);
			QModelIndex end = createIndex(item->row(), item->columnCount() - 1,
					rootItem);

			//emit dataChanged(begin, end);
			reset();
		}
	}
	return true;

}
bool ArrowTreeModel::appendRow(CPfeil &a) {
	//QModelIndex begin = createIndex(rootItem->childCount(),0, rootItem);
	//emit beginInsertRows(begin, rootItem->childCount(), rootItem->childCount());
	//qDebug() << __LINE__ << " " << __FILE__ << a.toString();

	this->m_list.push_back(a);
	std::list<CPfeil>::iterator iter = this->m_list.end();
	iter--;
	CPfeil *ptr = &(*(iter));
	TreeItem<CPfeil> *item = new TreeItem<CPfeil>(this->rootItem);
	this->create_data(item, this->rootItem, ptr, ptr->m_data);
	this->rootItem->childItems.append(item);

	//emit endInsertRows();
	reset();
	return true;

}
/**
 * @todo: the new element is only append to the list of arrows (m_list)
 * therefore m_list don't mirror the right order of elements!
 */
bool ArrowTreeModel::insertRow ( int row, const QModelIndex & parent )
{

	if(parent.isValid()){
		TreeItem<CPfeil> *item = static_cast<TreeItem<CPfeil>*> (parent.internalPointer());
		if (item == NULL)
			return false;
		if(item->parentItem != this->rootItem)
			return false;
	}else{
		row = 0;

	}

	CPfeil p;
	this->m_list.push_back(p);
	std::list<CPfeil>::iterator iter = this->m_list.end();
	iter--;
	CPfeil *ptr = &(*(iter));

	TreeItem<CPfeil> *new_item = new TreeItem<CPfeil>(this->rootItem);
	this->create_data(new_item, this->rootItem, ptr, ptr->m_data);
	this->rootItem->childItems.insert(row, new_item);
	return true;
}
bool ArrowTreeModel::insertRows ( int row, int count, const QModelIndex & parent  )
{
	QModelIndex index = parent;;
	if (!parent.isValid()){
		index = QModelIndex();
		row = 0;
	}

	emit beginInsertRows(index, row, row + count -1);
	for(int i=0;i<count; i++){
		if(!this->insertRow(row, index))
			return false;
	}
	emit endInsertRows();
	reset();
	return true;

}

void ArrowTreeModel::clear() {
	//qDeleteAll(this->rootItem->childItems);
	this->rootItem->base_item = NULL;
	this->rootItem->childItems.clear();
	reset();

}
bool ArrowTreeModel::removeRow ( int row, const QModelIndex & parent )
{
	if (!parent.isValid())
		return false;

	TreeItem<CPfeil> *item = static_cast<TreeItem<CPfeil>*> (parent.internalPointer());
	if (item == NULL)
		return false;
	if(item->parentItem != this->rootItem)
		return false;

	item = this->rootItem->childItems.value(row);
	if(item->base_item == NULL)
		return false;


	this->m_list.remove(*(item->base_item));
	this->rootItem->childItems.removeAt(row);
	return true;

}
bool ArrowTreeModel::removeRows ( int row, int count, const QModelIndex & parent  )
{
	if (!parent.isValid())
		return false;
	emit beginRemoveRows(parent, row, count);
	for(int i=0;i<count; i++){
		if(!this->removeRow(row, parent))
			return false;
	}
	emit endRemoveRows();
	return true;
}
