/***************************************************************************
 *   Copyright (C) 2010 by Herbert Roider                                  *
 *   herbert.roider@utanet.at                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/*
 * UnitInputWidget.cpp
 *  Created on: 21.12.2010
 */

#include "UnitInputWidget.h"
#include <QHBoxLayout>
#include <QDebug>


UnitInputWidget::UnitInputWidget(QWidget *parent , Qt::WindowFlags f ) {
	this->txtValue = new QLineEdit();
	this->cboUnit = new QComboBox();
	this->unitconverter = new UnitConverter();
	m_default_value = 0.0;
	m_has_default_value = false;

	this->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
	this->setMinimumHeight(this->cboUnit->baseSize().height() );

	QHBoxLayout *layout = new QHBoxLayout;
	layout->addWidget(cboUnit);
	layout->addWidget(txtValue);
	this->setLayout(layout);

	this->validator = new QDoubleValidator(this);
	this->validator->setNotation(QDoubleValidator::ScientificNotation);

	this->txtValue->setValidator(this->validator);

	this->status = UnitInputWidget::user;

	connect(this->cboUnit, SIGNAL(currentIndexChanged(int)), this,
			SLOT(on_cboUnit_currentIndexChanged(int)));

	connect(this->txtValue, SIGNAL(textEdited(const QString &)), this,
			SLOT(on_txtValue_textEdited(const QString &)));
	connect(this->txtValue, SIGNAL(editingFinished()), this,
			SLOT(on_txtValue_editingFinished()));

	this->style_calculated = "QLineEdit {color: rgb(0, 150, 0);}";
	this->style_preset = "QLineEdit {color: rgb(0, 0, 150);}";
	this->style_user = "";
	this->m_unitstyle = UnitInputWidget::name_symbol;
	// important because of the height
	this->layout()->setSizeConstraint(QLayout::SetMinimumSize);
	//this->layout()->update();
	this->updateGeometry();


}

UnitInputWidget::~UnitInputWidget() {

}
void UnitInputWidget::setText(QString value, QString unit_id, UnitConverter* conv, Value status  ) {

	if(conv != NULL) {
		this->unitconverter = conv;
	}
	this->m_unit_id = unit_id;

	QList<Unit> list = this->unitconverter->get_units(unit_id);
	this->cboUnit->clear();
	for (int i = 0; i < list.size(); i++) {
		this->cboUnit->addItem(this->getComboboxCaption(static_cast<Unit>(list.at(i))),
				list.at(i).m_type_id);

	}
	QVariant var(unit_id);
	int index = this->cboUnit->findData(var);
	this->cboUnit->setCurrentIndex(index);

	this->txtValue->setText(value);

	this->status = status;

	////////////////////////
	// Styles:
	switch (this->status) {
	case UnitInputWidget::calculated:
		this->txtValue->setStyleSheet(this->style_calculated);
		break;
	case UnitInputWidget::preset:
		this->txtValue->setStyleSheet(this->style_preset);
		break;
	case UnitInputWidget::user:
		this->txtValue->setStyleSheet(this->style_user);
		break;

	default:
		this->txtValue->setStyleSheet(this->style_user);
	}
	//this->updateGeometry();


}
void UnitInputWidget::setValue(double value, QString unit_id, UnitConverter* conv, Value status ) {
	QString text = QString().setNum(value);
	return this->setText(text, unit_id, conv, status);
}

void UnitInputWidget::on_cboUnit_currentIndexChanged ( int index ) {

	//qDebug() << "on_cboUnit_currentIndexChanged";
	QString strVal =  this->txtValue->text();
	if(strVal.length() == 0) {
		return;

	}
	if (index < 0)
		return;
	double value = strVal.toDouble();

	QString new_unit_id;
	this->get_unit_id(new_unit_id, index);

	this->unitconverter->convert(value, new_unit_id, this->m_unit_id);
	this->txtValue->setText(QString::number(value));
	this->m_unit_id = new_unit_id;
}
bool UnitInputWidget::get_unit_id(QString &unit_id, int index) {
	if (index < 0)
		return false;
	unit_id = this->cboUnit->itemData(index).toString();
	return true;
}
bool UnitInputWidget::set_unit_id(QString unit_id) {
	QVariant var(unit_id);
	int index = this->cboUnit->findData(var);
	this->cboUnit->setCurrentIndex(index);
	return true;

}
bool UnitInputWidget::setUnitId(QString new_unit_id) {
	QString oldUnitId = this->m_unit_id;
	if( ! this->set_unit_id(new_unit_id)) {
		return false;
	}
	return true;


}
void UnitInputWidget::on_txtValue_textEdited(const QString & text) {
	//qDebug() << "on_txtValue_textEdited: " << text;

	this->status = UnitInputWidget::user;

	// remove the validator if the textfield is empty, otherwise the signal editingFinished is not emited
	// @see on_txtValue_editingFinished
	if(text.trimmed().length() <= 0 && this->m_has_default_value) {
		this->txtValue->setValidator(NULL);
	}else{
		if( ! this->txtValue->validator()) {
			this->txtValue->setValidator(this->validator);
		}

	}

	////////////////////////
	// Styles:
	switch (this->status) {
	case UnitInputWidget::calculated:
		this->txtValue->setStyleSheet(this->style_calculated);
		break;
	case UnitInputWidget::preset:
		this->txtValue->setStyleSheet(this->style_preset);
		break;
	case UnitInputWidget::user:
		this->txtValue->setStyleSheet(this->style_user);
		break;

	default:
		this->txtValue->setStyleSheet(this->style_user);
	}

}
void UnitInputWidget::on_txtValue_editingFinished() {
	//qDebug() << "	void on_txtValue_editingFinished();";

	QString text = this->txtValue->text();

	if(text.trimmed().length() <= 0 && this->m_has_default_value) {
		this->displayDefaultValue();

	}


	//emit editingFinished();

}

QString UnitInputWidget::getComboboxCaption(const Unit &u) {
	QString caption;
	if(m_unitstyle == UnitInputWidget::name) {
		caption = u.m_name;

	}else if(m_unitstyle == UnitInputWidget::symbol) {
		caption = u.m_symbol;

	}else if(m_unitstyle == UnitInputWidget::name_symbol) {
		caption = u.m_name + QString(" [") + u.m_symbol + QString("]");

	}else if(m_unitstyle == UnitInputWidget::symbol_in_bracket) {
		caption = QString("[") + u.m_symbol + QString("]");

	}
	return caption;

}
double UnitInputWidget::getSIValue(bool & ok) {
	QString str = this->txtValue->text();
	ok = false;
	if(str.isEmpty()) {
		ok = false;
		return 0.0;
	}
	double value = str.toDouble(&ok);
	if(! ok) {
		return 0.0;
	}
	this->unitconverter->convert_to_SI(value, this->m_unit_id);
	return value;
}
/**
* @brief
*
*  @param   double    value in SI unit
*
*  @return  void
*
*/
void UnitInputWidget::setDefaultValue(double v) {
	this->m_default_value = v;
	this->m_has_default_value = true;

}
void UnitInputWidget::unsetDefaultValue() {
	this->m_default_value = 0.0;
	this->m_has_default_value = false;


}
/**
* @brief show the default value in the textfield. The value will be converted to the current unit.
*
*  @param
*
*  @return  void
*  */
void UnitInputWidget::displayDefaultValue() {
	if( ! this->m_has_default_value)
		return;
	//this->status = UnitInputWidget::preset;
	QString unit_id = this->getUnitId();
	double value = this->m_default_value;
	this->unitconverter->convert_from_SI(value, unit_id);
	this->setValue(value, unit_id, NULL, UnitInputWidget::preset);
}

