/***************************************************************************
 *   Copyright (C) 2010 by Herbert Roider                                  *
 *   herbert.roider@utanet.at                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/*
 * UnitInputWidget.h
 *  Created on: 21.12.2010
 */

#ifndef UNITINPUTWIDGET_H_
#define UNITINPUTWIDGET_H_

#include <QWidget>
#include <QLineEdit>
#include <QComboBox>
#include "UnitConverter.h"
#include <QDoubleValidator>

class UnitInputWidget :  public QWidget {


	Q_OBJECT
	Q_PROPERTY(Unitstyle unitstyle READ unitstyle WRITE setUnitstyle )
	Q_ENUMS(Unitstyle)


	UnitConverter* unitconverter;
	QLineEdit* txtValue;
	QComboBox* cboUnit;
	QString m_unit_id;
	QDoubleValidator* validator ;


	bool get_unit_id(QString &unit_id, int index) ;
	bool set_unit_id(QString unit_id);

	QString style_calculated ;
	QString style_preset ;
	QString style_user;

	double m_default_value;
	bool m_has_default_value;




public:
	enum Value {
		preset = 0,
		user = 1 ,
		calculated = 2

	} ;
	enum Unitstyle {
		name = 0,
	    symbol = 1,
	    name_symbol = 2,
	    symbol_in_bracket = 3

	};

	Value getStatus() {
		return this->status;
	};
	void setUnitstyle(const Unitstyle _unitstyle) { this->m_unitstyle = _unitstyle;  };
	Unitstyle unitstyle() const { return this->m_unitstyle; };

	UnitInputWidget(QWidget *parent = 0, Qt::WindowFlags f = 0 );
	virtual ~UnitInputWidget();

	void setText(QString value, QString unit_id, UnitConverter* conv, Value status  );
	void setValue(double value, QString unit_id, UnitConverter* conv, Value status);
	QString text() {
		return txtValue->text();
	};
	QString getUnitId() {
		return m_unit_id;
	}
	double getSIValue(bool & ok);
	bool setUnitId(QString new_unit_id);
	void setValidator ( const QValidator * v ) {
		if(v == NULL) {
			return;
		}
		this->txtValue->setValidator(v);
	};
	QString getComboboxCaption(const Unit &u);

	void setDefaultValue(double v);
	void unsetDefaultValue();
	void displayDefaultValue();
private:
	Value status;
	Unitstyle m_unitstyle;


public slots:
	void on_cboUnit_currentIndexChanged ( int index );
	void on_txtValue_textEdited(const QString & text);
	void on_txtValue_editingFinished();


signals:
	void editingFinished();
	void currentIndexChanged( int index );


};

#endif /* UNITINPUTWIDGET_H_ */
