/**
 Changes by Herbert Roider
 - move the functins of the file utils.c in this file

pack all the functions in a class "aspline"
The functions are based on the functions from David Frey

 */
/***************************************************************************
 * aspline.c -- aspline does an Akima spline interpolation.                *
 *                                                                         *
 * spline is (c) David Frey, 1996, 1998, 1999, 2002                        *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify it *
 * under the terms of the GNU General Public License as published by the   *
 * Free Software Foundation; either version 2 of the License, or (at your  *
 * option) any later version.                                              *
 *                                                                         *
 * This program is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
 * General Public License for more details.                                *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with this program; if not, write to the Free Software Foundation, *
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA            *
 ***************************************************************************/
/***************************************************************************
 *   Copyright (C) 2009 by Herbert Roider                                  *
 *   herbert.roider@utanet.at                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *

 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include <float.h>
#include <string.h>
#include <ctype.h>
#include <getopt.h>
#include <errno.h>


/** add by Herbert Roider */
#include "aspline.h"
#include <vector>
struct POINT {
	double x;
	double y;
};

void aspline::computelimits(int s, int e, double *llimit, double *ulimit) {
	int i;

	*llimit = FLT_MAX;
	*ulimit = -FLT_MAX;
	for (i = s; i < e; i++) {
		if (x[i] < *llimit)
			*llimit = x[i];
		if (x[i] > *ulimit)
			*ulimit = x[i];
	}
}

aspline::aspline() {
	C = NULL;
	D = NULL;
	x = NULL;
	y = NULL;
	t = NULL;

	clear();

}
aspline::aspline(const aspline &s)
{
	C = NULL;
	D = NULL;
	t = NULL;
	x = NULL;
	y = NULL;

	clear();

	this->err = s.err;
	this->n = s.n;
	this->n_memory = s.n_memory;
	this->not_calculatet = s.not_calculatet;

	x = new double [n_memory];
	y = new double [n_memory];
	C = new double [n];
	D = new double [n];
	t = new double [n];

	memmove(x, s.x, n * sizeof(double) );
	memmove(y, s.y, n * sizeof(double) );
	memmove(C, s.C, n * sizeof(double) );
	memmove(D, s.D, n * sizeof(double) );
	memmove(t, s.t, n * sizeof(double) );


}
aspline & aspline::operator=(const aspline & s)
{
	if(this == &s)
		return *this;
	clear();

	this->err = s.err;
	this->n = s.n;
	this->n_memory = s.n_memory;
	this->not_calculatet = s.not_calculatet;

	x = new double [n_memory];
	y = new double [n_memory];
	C = new double [n];
	D = new double [n];
	t = new double [n];

	memmove(x, s.x, n * sizeof(double) );
	memmove(y, s.y, n * sizeof(double) );
	memmove(C, s.C, n * sizeof(double) );
	memmove(D, s.D, n * sizeof(double) );
	memmove(t, s.t, n * sizeof(double) );

	return *this;

}

aspline::~aspline() {
	if (C != 0)
		delete [] C ;
	if (D != 0)
		delete [] D;
	if (t != 0)
		delete [] t;
	C=NULL;
	D=NULL;
	t=NULL;
	clear();
}
void aspline::clear() {
	n = 0;
	n_memory = 0;
	not_calculatet = true;
	err = false;


	if (x != 0)
		delete [] x;
	if (y != 0)
		delete [] y;

	x = NULL;
	y = NULL;
}
int aspline::insertpoint(double xv, double yv) {
	double * control_points_x;
	double * control_points_y;
	int n_control_points;
	err = false;
	not_calculatet = true;


	int k;
	// allocate memory if necessary:
	if( (n + 10) > n_memory ){
		double  *x_old = x;
		double  *y_old = y;
		n_memory = n + 5;
		x = new double [n_memory];
		y = new double [n_memory];
		if(n > 0){
			memmove(x, x_old, n * sizeof(double) );
			memmove(y, y_old, n * sizeof(double) );
		}
		delete [] x_old;
		delete [] y_old;

	}
	/** The first 2 points and the last 2 points are used for calculation, @see calcspline
	 *
	 */
	if(n == 0)
		n = 4; //  The first 2 points and the last 2 points are used for calculation, @see calcspline
	n_control_points = n-4;
	control_points_x = &x[2];
	control_points_y = &y[2];

	/* search the point */
	k = 0;
	while ((k < n_control_points) && (xv > control_points_x[k]))
		k++;

	if (k == n_control_points) { /* new point */

		control_points_x[k] = xv;
		control_points_y[k] = yv;
		(n)++;
	} else if (xv == control_points_x[k]) { /* duplicate */
		control_points_y[k] = yv; // by Herbert Roider: set the new y-value for a already existing point
	} else { /* intermediate point. Insert it */
		/* Move the points above */
		memmove(&control_points_x[k + 1], &control_points_x[k], (size_t)((n_control_points - k) * sizeof(double)));
		memmove(&control_points_y[k + 1], &control_points_y[k], (size_t)((n_control_points - k) * sizeof(double)));

		control_points_x[k] = xv;
		control_points_y[k] = yv;
		(n)++;
	}
	return 0;
}
int aspline::calcspline() {
	// less than 3 control points plus 4 additional points:
	err = false;
	if(n < 7){
		err = true;
		return -1;
	}


	int i; /* n=no of points */

	/* Leading extrapolation points,
	 actual values will be filled in later */

	/* Move x[0] to x[2] */
	//memmove(&x[2], &x[0], (size_t)(n * sizeof(double)));
	//memmove(&y[2], &y[0], (size_t)(n * sizeof(double)));
	//n += 4;

	/* calculate coefficients of the spline (Akima interpolation itself) */

	/* allocate the arrays
	 * NB: There are some unused locations (dx[0]),
	 *     but this faciliates the indexing.
	 */
	double *dx = new double [n];
	double *dy = new double [n];
	double *m = new double [n]; //slope

	if (C != 0)
		delete [] C ;
	if (D != 0)
		delete [] D;
	if (t != 0)
		delete [] t;
	C = new double [n];
	D = new double [n];
	t = new double [n];


	/* a) Calculate the differences and the slopes m[i]. */

	for (i = 2; i < n - 3; i++) {
		dx[i] = x[i + 1] - x[i];
		dy[i] = y[i + 1] - y[i];
		m[i] = dy[i] / dx[i]; /* dx != 0, asserted by insertpoint() */
	}

	/* b) interpolate the missing points: */

	x[1] = x[2] + x[3] - x[4];
	dx[1] = x[2] - x[1];
	y[1] = dx[1] * (m[3] - 2.0 * m[2]) + y[2];
	dy[1] = y[2] - y[1];
	m[1] = dy[1] / dx[1];

	x[0] = 2.0 * x[2] - x[4];
	dx[0] = x[1] - x[0];
	y[0] = dx[0] * (m[2] - 2.0 * m[1]) + y[1];
	dy[0] = y[1] - y[0];
	m[0] = dy[0] / dx[0];

	x[n - 2] = x[n - 3] + x[n - 4] - x[n - 5];
	y[n - 2] = (2.0 * m[n - 4] - m[n - 5]) * (x[n - 2] - x[n - 3])
			+ y[n - 3];

	/** bugfix by Herbert Roider
	 *  m[n-3] was not initialised!
	 */
	dx[n-3] = x[n-2] - x[n-3];
	dy[n-3] = y[n-2] - y[n-3];
	m[n-3] = dy[n-3] / dx[n-3]; // this value was not set!
	///////////////////////////////////

	x[n - 1] = 2.0 * x[n - 3] - x[n - 5];
	y[n - 1] = (2.0 * m[n - 3] - m[n - 4]) * (x[n - 1] - x[n - 2])
			+ y[n - 2];

	/** bugfix by Herbert Roider
	 *
	 */
	dx[n-2] = x[n-1] - x[n-2];
	dy[n-2] = y[n-1] - y[n-2];
	m[n-2] = dy[n-2] / dx[n-2];

// edit by Herbert Roider
// this is no more needed:
//	for (i = n - 3; i < n - 1; i++) {
//		dx[i] = x[i + 1] - x[i];
//		dy[i] = y[i + 1] - y[i];
//		m[i] = dy[i] / dx[i];
//	}
	///////////////////////////////////

	/* the first x slopes and the last y ones are extrapolated: */

	t[0] = 0.0;
	t[1] = 0.0; /* not relevant */

	for (i = 2; i < n - 2; i++) {
		double num, den;

		num = fabs(m[i + 1] - m[i]) * m[i - 1] + fabs(m[i - 1] - m[i
				- 2]) * m[i];
		den = fabs(m[i + 1] - m[i]) + fabs(m[i - 1] - m[i - 2]);

		if (den != 0.0)
			t[i] = num / den;
		else
			t[i] = 0.0;
	}

	/* c) Allocate the polynom coefficients */

	/* A=y_i
	 B=t_i */
/** bugfix by Herbert Roider:
 * change "i < n-2" to "i < n-3" because t[i+1] contains no value when i==n-3
 */
	for (i = 2; i < n - 3; i++) {
		/* A[i]=y[i];
		 B[i]=t[i]; */
		C[i] = (3.0 * m[i] - 2.0 * t[i] - t[i + 1]) / dx[i];
		D[i] = (t[i] + t[i + 1] - 2.0 * m[i]) / (dx[i] * dx[i]);
	}
	delete [] dx;
	delete [] dy;
	delete [] m ;



	not_calculatet = false;


	return 0;
}
int aspline::spline(double & yv, double xv) {
	yv = 0.0;
	if(err)
		return -1;
	if (not_calculatet == true){
		return -2;
	}
	/* 3rd step: output the coefficients for the subintervalls i=2..n-4 */
	/* calculate the intermediate values */

	int k = 2;

	while (xv >= x[k] && k < n-2) {
		k++; /* XXX */
	};
	k--;
	if(k > n-4)
		k = n-4;
	if(k < 2)
		k = 2;


	yv = y[k] + t[k] * (xv - x[k]) + C[k] * pow((xv - x[k]), 2.0) + D[k] * pow(
			(xv - x[k]), 3.0);

	return 0;

}

