/***************************************************************************
 *   Copyright (C) 2010 by Herbert Roider   *
 *   herbert.roider@utanet.at   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/*
 * calc_inclination_of_sight_dlg.cpp
 *  Created on: 30.07.2009
 */


#include "calc_inclination_of_sight_dlg.h"
#include <QMessageBox>
#include "ArrowTreeModel.h"
#include <QTextStream>


calc_inclination_of_sight_dlg::calc_inclination_of_sight_dlg(QWidget *parent) :
	calc_velocity_dlg(parent) {
	ui.lbl_wide_setting_y->hide();
	ui.lbl_wide_setting_z->hide();
	ui.txt_wide_setting_y->hide();
	ui.txt_wide_setting_z->hide();
	ui.selReticle_wide->hide();
	ui.lbl_wide_devitation_y->hide();
	ui.lbl_wide_setting_z_2->hide();
	ui.txt_wide_devitation_y->hide();
	ui.txt_wide_devitation_z->hide();
	ui.lbl_wide_range->hide();
	ui.lbl_small_range->hide();
	ui.txt_wide_range->hide();
	ui.line->hide();

	//ui.setupUi(this);
}
void calc_inclination_of_sight_dlg::set_values(CArmbrust &a) {

	calc_velocity_dlg::set_values(a);
	//ui.grp_wide_range->hide();
	//ui.grp_small_range->setTitle(QString(""));

}
void calc_inclination_of_sight_dlg::init(CArmbrust &a, ArrowTreeModel *p) {
	calc_velocity_dlg::init(a,p);
	set_values(m_crossbow);

}

void calc_inclination_of_sight_dlg::on_cmdCalc_clicked() {
	CArmbrust a = m_crossbow;
	get_values(a);

	bool ok;
	double small_range = ui.txt_small_range->getSIValue(ok);
	double wide_range = ui.txt_wide_range->getSIValue(ok);

	utility::vector_3d setting_small;
	utility::vector_3d setting_wide;

	utility::vector_3d devitation_small;
	devitation_small.x = 0.0;
	devitation_small.y = ui.txt_small_devitation_y->getSIValue(ok);
	devitation_small.z = ui.txt_small_devitation_z->getSIValue(ok);

	// vector points from the targetpoint to the impactpoint (x = 0)
	utility::vector_3d devitation_wide;
	devitation_wide.x = 0.0;
	devitation_wide.y = ui.txt_wide_devitation_y->getSIValue(ok);
	devitation_wide.z = ui.txt_wide_devitation_z->getSIValue(ok);

	if (a.m_Zielgeraet.m_iTyp == BENUTZERDEF) {
		bool ok_small, ok_wide;
		setting_small = ui.selReticle_small->getSetting(ok_small);
		setting_wide = ui.selReticle_wide->getSetting(ok_wide);
		if( ! (ok_small )){
			QMessageBox::information(this, tr("Error"), tr("select line of the reticle"));//MessageBox("einen Pfeil aus der Pfeilliste ausw�len!", "Arrowmatcher", MB_ICONEXCLAMATION);
			return;
		}

	} else if (a.m_Zielgeraet.m_iTyp == KIMME_U_KORN) {
		setting_small.x = 1.0;
		setting_small.y = ui.txt_small_setting_y->text().toDouble()
				* a.m_Zielgeraet.m_graduation.y;
		setting_small.z = ui.txt_small_setting_z->text().toDouble()
				* a.m_Zielgeraet.m_graduation.z;

		setting_wide.x = 1.0;
		setting_wide.y = ui.txt_wide_setting_y->text().toDouble()
				* a.m_Zielgeraet.m_graduation.y;
		setting_wide.z = ui.txt_wide_setting_z->text().toDouble()
				* a.m_Zielgeraet.m_graduation.z;
	}

	QItemSelectionModel *selection_model = ui.lst_arrows->selectionModel();
	if (selection_model == NULL) {
		QMessageBox::information(this, tr("Error"), tr("select an arrow"));//MessageBox("einen Pfeil aus der Pfeilliste ausw�len!", "Arrowmatcher", MB_ICONEXCLAMATION);
		return;
	}
	QModelIndex index = selection_model->currentIndex();

	CPfeil *p_arrow = m_arrow_list_model->getItem(index);//= ui.lstPfeile->GetSelPfeil();
	if (p_arrow == NULL) {
		QMessageBox::information(this, tr("Error"), tr("select an arrow"));//MessageBox("einen Pfeil aus der Pfeilliste ausw�len!", "Arrowmatcher", MB_ICONEXCLAMATION);
		return;
	}

	flightparam fp;
	Ballistic::flightparam_init(fp, *p_arrow);
	Ballistic::flightparam_gravity(fp, 0.0);
	Ballistic::flightparam_wind(fp, 0.0, 0.0);

	double v;
	utility::vector_3d scso_rot;
	int err = 0;

	err = a.calc_sight_inclination(	scso_rot, small_range, setting_small, devitation_small, fp);

	//err = a.calc_velocity(v, scso_rot, small_range, wide_range, setting_small,
	//		setting_wide, devitation_small, devitation_wide, fp);
	if (err) {
		QMessageBox::critical(this, tr("Error"), tr("Cannot calculate speed"),
				QMessageBox::Ok);
		return;
	}

	QString unitstr ;
	int status ;
	UnitConverter uv;

	unitstr = a.getValue("scso/rot/y/unit").toString();
	status = a.getValue("scso/rot/y/status").toInt();
	v = scso_rot.y;
	uv.convert_from_SI(v, unitstr);
	a.setValue("scso/rot/y/status", (int)Datastore::user );
	a.setValue("scso/rot/y/value", v);

	unitstr = a.getValue("scso/rot/z/unit").toString();
	status = a.getValue("scso/rot/z/status").toInt();
	v = scso_rot.z;
	uv.convert_from_SI(v, unitstr);
	a.setValue("scso/rot/z/status", (int)Datastore::user );
	a.setValue("scso/rot/z/value", v);



	m_crossbow = a;
	//set_values(m_crossbow);
	print_result(m_crossbow, *p_arrow);

}
void calc_inclination_of_sight_dlg::print_result(CArmbrust &a, CPfeil &p) {
	QString s;
	QTextStream outputstream(&s);

	outputstream << "<table cellpadding=0 cellspacing=0><tr><td>"
			<< tr("Angle") << "</td><td>" << "<table>"
			<< "<tr><td>Z&nbsp;" << (a.m_scso.rot.z*1000.0) << "</td></tr>"
			<< "<tr><td>Y&nbsp;" << (a.m_scso.rot.y*1000.0) << "</td></tr>"
			<< "</table></td><td>" << tr(
			"millirad") << "</td></tr>";

	outputstream << "</table>";
	ui.lbl_solution->setText(s);

}
